==> snf_3.3.0_webrelease.tar.gz: snf_3.3.0_webrelease/src/snfdc/g98rslave.r <==
CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine g98rslave(grad,co,ilab,nmode,freq,intonly,nummodes)
C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
C written by C. Herrmann, Uni Jena, 05/2005
C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      include 'defs.h'
      include 'comm.h'
      include 'param.h'  

      integer ibuf,info,nn,iat,idum,ixyz,irc,idir,l,mypid
      integer idmy,j,mbuf,diraus,ladf,iclean,icleanl
      real*8 escf,gsum
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)

      integer lprop,lpol,ldscf,lgrad,kstep,xprog,ldmol
      integer xprogb
      integer*4 itst,jstepa
      character*255 b1
      character*5 nic
      character*24 date,dmname
      character*132 forceout,polout
      real*8 cpu,wall,gcpu,gwall
      integer nit
      integer xeq,tmpxeq,icmp
      real*8 grad(nvar),dipol(3),co(3,natoms),polar(8,mxomeg)
      real*8 gtenlao(9,mxomeg),gtenao(9,mxomeg),aten(27,mxomeg)
      real*8 dipal(3),dipbe(3)
      integer ilab(natoms),lfout,lpolo
      logical found,snffast
C#    integer w,st(MPI_STATUS_SIZE)
C#    data w /MPI_COMM_WORLD/
      integer intonly,nummodes,i

      nic='nice '
      kstep=1

*-------------------
* call rslave module
*-------------------
      call rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes)

*------------------------------------------------------------------
* do the calculation
*-------------------------------------------------------------------
      if(snffast) then
        nic='     '
        write (iosl,*) '** found $snffast flag: omitting command nice'
      end if
      call flush(iosl)
      if(snffast) then
        nic='     '
        write (iosl,*) 'test calculation: omitting command nice'
      end if
*--------------------------
* prepare in-/output file names
*--------------------------
      CALL augnum(forceout,'force.log.',jstepa)
      CALL llen(forceout,lfout)
      if(calcno.eq.1) then
        write(iosl,*) '** rslavem30: running gaussian98'
        call flush(iosl)
        info=xprog(nic(1:5)//progdir(1:lb)//'g98 ' //tempdir(1:lt)//
     *  '/force.com','force.log')
      else if(calcno.eq.2) then
         write(iosl,*) '** rslavem30: running gaussian03'
        call flush(iosl)
        info=xprog(nic(1:5)//progdir(1:lb)//'g03 ' //tempdir(1:lt)//
     *  '/force.com','force.log')
      end if

      CALL ggaussgrad(natoms,escf,dipol,grad,tempdir(1:lt)//'/'
     *                //'force.log')
      write(iosl,*) '** rslavem30: gradient read'
      CALL flush(iosl)

      info=system('mv '//tempdir(1:lt)//'/force.log '
     *                //tempdir(1:lt)//'/'//forceout(1:lfout))
      write(iosl,*)('rslavem30: mv '//tempdir(1:lt)//'/force.log '
     *                //tempdir(1:lt)//'/'//forceout(1:lfout))
      CALL flush(iosl)
      if(info.ne.0) call errex('rslavem30: system command failed')
      CALL flush(iosl)

*------------------------------------------------------------
* get polarizabilities:
* 1) prepare input file for single-point calculation
* 2) perform single-point calculation
* 4) extract components of polarizability from output
*------------------------------------------------------------
      if(ipol.ne.0) then
        call augnum(polout,'polar.log.',jstepa)
        call llen(polout,lpolo)
        call wgausspol(tempdir(1:lt),'polar.com',iosl)
        if(calcno.eq.1) then
          info=xprog(nic(1:5)//progdir(1:lb)//'g98 ' //tempdir(1:lt)//
     *     '/polar.com','polar.log')
        else if(calcno.eq.2) then
          info=xprog(nic(1:5)//progdir(1:lb)//'g03 ' //tempdir(1:lt)//
     *     '/polar.com','polar.log')
        end if
        write(iosl,*) 'POLAR.LOG = ', tempdir(1:lt)//'/polar.log'
        call flush(iosl)
        call ggausspol(polar,tempdir(1:lt)//'/polar.log',iosl)
        write(iosl,*) 'polar = ', (polar(i,1),i=1,6)
        info=system('mv '//tempdir(1:lt)//'/polar.log '
     *                //tempdir(1:lt)//'/'//polout(1:lpolo))
        write(iosl,*) 'rslavem30: mv '//tempdir(1:lt)//'/polar.log '
     *                //tempdir(1:lt)//'/'//polout(1:lpolo) 
        CALL flush(iosl)
        if(info.ne.0) call errex('rslavem30: system command failed')
        CALL flush(iosl)
      end if

      cpu=0.0d0
      wall=0.0d0
      gcpu=0.0d0
      gwall=0.0d0
      CALL rxxxx('force.log',nit,cpu,wall)
      WRITE (iosl,*) 'iterations: ',nit
      WRITE (iosl,*) 'cpu       : ',cpu
      WRITE (iosl,*) 'wall      : ',wall
      gsum=0.0d0
      do j=1,nvar    
         gsum=gsum+grad(j)**2
      enddo
      write (iosl,*) 'grad norm',gsum
      call flush(iosl)

*-------------------
* send final results
*-------------------
C:      call MPI_SSEND(nit,1,MPI_INTEGER,tid0,fsys_restag,w,info)
C:      call mpierr('MPI_SSEND nit','rslave',info)
C:      call MPI_SSEND(cpu,1,MPI_DOUBLE_PRECISION,tid0,801,w,info)
C:      call mpierr('MPI_SSEND cpu','rslave',info)
C:      call MPI_SSEND(wall,1,MPI_DOUBLE_PRECISION,tid0,802,w,info)
C:      call mpierr('MPI_SSEND wall','rslave',info)
C:      call MPI_SSEND(gcpu,1,MPI_DOUBLE_PRECISION,tid0,803,w,info)
C:      call mpierr('MPI_SSEND gcpu','rslave',info)
C:      call MPI_SSEND(gwall,1,MPI_DOUBLE_PRECISION,tid0,804,w,info)
C:      call mpierr('MPI_SSEND gwall','rslave',info)
C:      call MPI_SSEND(escf,1,MPI_DOUBLE_PRECISION,tid0,805,w,info)
C:      call mpierr('MPI_SSEND out1','rslave',info)
C:      call MPI_SSEND(dipol,3,MPI_DOUBLE_PRECISION,tid0,806,w,info)
C:      call mpierr('MPI_SSEND dipol','rslave',info)
C:      call MPI_SSEND(grad,nvar,MPI_DOUBLE_PRECISION,
C:   *   tid0,807,w,info)
C:      call mpierr('MPI_SSEND escf','rslave',info)
C:      IF (ipol.GT.0) THEN
C:       call MPI_SSEND(polar,8*ipol,MPI_DOUBLE_PRECISION,
C:   *    tid0,808,w,info)
C:       call mpierr('MPI_SSEND polar','rslave',info)
C:      end if
C//SKIPC:
C#      call MPI_SEND(nit,1,MPI_INTEGER,tid0,fsys_restag,w,info)
C#      call mpierr('MPI_SEND nit','rslave',info)
C#      call MPI_SEND(cpu,1,MPI_DOUBLE_PRECISION,tid0,801,w,info)
C#      call mpierr('MPI_SEND cpu','rslave',info)
C#      call MPI_SEND(wall,1,MPI_DOUBLE_PRECISION,tid0,802,w,info)
C#      call mpierr('MPI_SEND wall','rslave',info)
C#      call MPI_SEND(gcpu,1,MPI_DOUBLE_PRECISION,tid0,803,w,info)
C#      call mpierr('MPI_SEND gcpu','rslave',info)
C#      call MPI_SEND(gwall,1,MPI_DOUBLE_PRECISION,tid0,804,w,info)
C#      call mpierr('MPI_SEND gwall','rslave',info)
C#      call MPI_SEND(escf,1,MPI_DOUBLE_PRECISION,tid0,805,w,info)
C#      call mpierr('MPI_SEND escf','rslave',info)
C#      call MPI_SEND(dipol,3,MPI_DOUBLE_PRECISION,tid0,806,w,info)
C#      call mpierr('MPI_SEND dipol','rslave',info)
C#      call MPI_SEND(grad,nvar,MPI_DOUBLE_PRECISION,
C#   *   tid0,807,w,info)
C#      call mpierr('MPI_SEND grad','rslave',info)
C#      if (ipol.GT.0) then
C#       call MPI_SEND(polar,8*ipol,MPI_DOUBLE_PRECISION,
C#   *    tid0,808,w,info)
C#       call mpierr('MPI_SEND polar','rslave',info)
C#      end if
C//END
C//SKIPC#
      call PVMFINITSEND(PVMDATADEFAULT,MBUF)
      call PVMFPACK(INTEGER4,nit,kstep,1,info)
      call PVMFPACK(REAL8,cpu,kstep,1,info)
      call PVMFPACK(REAL8,wall,kstep,1,info)
      call PVMFPACK(REAL8,gcpu,kstep,1,info)
      call PVMFPACK(REAL8,gwall,kstep,1,info)
      call PVMFPACK(REAL8,escf,1,1, INFO )
      call PVMFPACK(REAL8,dipol,3,1,INFO )
      call PVMFPACK(REAL8,grad,nvar,1,INFO )
      IF (ipol.GT.0) then 
        write(iosl,*) 'packing polar'
        write(iosl,*) 'polar: ', (polar(i,1),i=1,6)
        call PVMFPACK(REAL8,polar,8*ipol,1,INFO )
      end if
      call PVMFSEND( TID0, 0, INFO )
C//END
      call mnotify(5,mypid,9,'results sent',12)
      return
      end

==> snf-4.0.0_webrelease.tar.gz: snf-4.0.0_webrelease/src/snfdc/g98rslave.r <==
CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine g98rslave(grad,co,ilab,nmode,freq,intonly,nummodes)

c     ====================================================================
c     purpose: perform a GAUSSIAN single point calculation for a displaced
c              structure, collect the results, and send them to master
c 
c     history:
c       - written by C. Herrmann, Uni Jena, 05/2005
c       - modified by C. Herrmann, 07/2006
c       - cleaned and modified by J. Neugebauer, ETH Zurich, 2007
c     ====================================================================

      include 'defs.h'
      include 'param.h'  

      integer info,iat,l
      integer j,diraus,iclean,icleanl,system
      real*8 escf
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)

      integer xprogb,xprog
      integer*4 jstepa
      character*255 b1
      character*5 nic
      character*255 forceout,forcein,polout
      real*8 cpu,wall,gcpu,gwall
      integer*4 nit
      integer xeq,tmpxeq
      real*8 grad(nvar),dipol(3),co(3,natoms),polar(8,mxomeg)
      real*8 gtenlao(9,mxomeg),gtenao(9,mxomeg),aten(27,mxomeg)
      integer ilab(natoms),lfout,lfin,lpolo
      logical found
      integer intonly,nummodes,i

*-------------------
* call rslave module
*-------------------
      call rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes,nic)

*--------------------------
* prepare in-/output file names
*--------------------------
      CALL augnum(forcein,'force.com.',jstepa)
      CALL llen(forcein,lfin)
      info=system('mv '//tempdir(1:lt)//'/force.com '//tempdir(1:lt)//
     *  '/'//forcein(1:lfin))
      CALL augnum(forceout,'force.log.',jstepa)
      CALL llen(forceout,lfout)
      if(calcno.eq.1) then
        write(iosl,*) '** rslavem30: running gaussian98'
        call flush(iosl)
        info=xprog(nic(1:5)//progdir(1:lb)//'g98 < '//tempdir(1:lt)//
     *  '/'//forcein(1:lfin),tempdir(1:lt)//'/'//forceout(1:lfout))
      else if(calcno.eq.2) then
         write(iosl,*) '** rslavem30: running gaussian03'
        call flush(iosl)
        info=xprog(nic(1:5)//progdir(1:lb)//'g03 < ' //tempdir(1:lt)//
     *  '/'//forcein(1:lfin),tempdir(1:lt)//'/'//forceout(1:lfout))
      end if

      CALL ggaussgrad(natoms,escf,dipol,grad,tempdir(1:lt)//'/'
     *                //forceout(1:lfout))
      write(iosl,*) '** rslavem30: gradient read'
      CALL flush(iosl)

      if(info.ne.0) call errex('rslavem30: system command failed')
      CALL flush(iosl)

*------------------------------------------------------------
* get polarizabilities:
* 1) prepare input file for single-point calculation
* 2) perform single-point calculation
* 4) extract components of polarizability from output
*------------------------------------------------------------
      if(ipol.ne.0) then
        call augnum(polout,'polar.log.',jstepa)
        call llen(polout,lpolo)
        call wgausspol(tempdir(1:lt),forcein(1:lfin),'polar.com',iosl)
        if(calcno.eq.1) then
          info=xprog(nic(1:5)//progdir(1:lb)//'g98 ' //tempdir(1:lt)//
     *     '/polar.com','polar.log')
        else if(calcno.eq.2) then
          info=xprog(nic(1:5)//progdir(1:lb)//'g03 ' //tempdir(1:lt)//
     *     '/polar.com','polar.log')
        end if
        call ggausspol(polar,tempdir(1:lt)//'/polar.log',iosl)
        info=system('mv '//tempdir(1:lt)//'/polar.log '
     *                //tempdir(1:lt)//'/'//polout(1:lpolo))
        write(iosl,*) 'rslavem30: mv '//tempdir(1:lt)//'/polar.log '
     *                //tempdir(1:lt)//'/'//polout(1:lpolo) 
        CALL flush(iosl)
        if(info.ne.0) call errex('rslavem30: system command failed')
        CALL flush(iosl)
      end if

      cpu=0.0d0
      wall=0.0d0
      gcpu=0.0d0
      gwall=0.0d0

c     ------------------
c     send final results
c     ------------------
      call sendres(nit,cpu,wall,gcpu,gwall,escf,dipol,grad,
     *   polar,gtenlao,gtenao,aten,jstepa,nummodes,intonly)

      return
      end
